#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/init.h>
#include <linux/config.h>
#include <linux/version.h>
#include <linux/interrupt.h>

#define MAX_NR_CONSOLES 1

/*** copy from console.c ---> ***/
/*
 * fg_console is the current virtual console,
 * last_console is the last used one,
 * want_console is the console we want to switch to,
 * kmsg_redirect is the console for kernel messages,
 */
int fg_console = 0;
int last_console = 0;
int want_console = -1;
int kmsg_redirect = 0;
/*** <--- copy from console.c ***/

/*** copy from console.c ---> ***/
/*
 * Unfortunately, we need to delay tty echo when we're currently writing to the
 * console since the code is (and always was) not re-entrant, so we insert
 * all filp requests to con_task_queue instead of tq_timer and run it from
 * the console_bh.
 */
DECLARE_TASK_QUEUE(con_task_queue);
/*** <--- copy from console.c ***/


/*
 *	/dev/ttyN handling
 */

extern void zaucons_write(const char*,unsigned int);

static int zaucon_write(struct tty_struct * tty, int from_user,
		     const unsigned char *buf, int count)
{
	int i;

	printk("%s:%d (%s)\n", __FILE__, __LINE__, __FUNCTION__);
#if 1
	zaucons_write(buf,count);
#endif
#if 0
	printk("[cw%d<",count);
	for(i = 0; i < count; i++){
		if(i != 0){
			printk(" ");
		}
		printk("%02x",buf[i]);
	}
	printk(">");
	for(i = 0; i < count; i++){
		int	ch;
		ch = buf[i];
		printk("%c",ch>=32&&ch<=126?ch:'.');
	}
	printk("]");
#endif
	return count;
}

static void zaucon_put_char(struct tty_struct *tty, unsigned char ch)
{
#if 1
	zaucons_putchar(ch);
#endif
#if 0
	printk("[co%02x%c]",ch,ch>=32&&ch<=126?ch:'.');
#endif
}

static int zaucon_write_room(struct tty_struct *tty)
{
	if (tty->stopped)
		return 0;
	return 4096;		/* No limit, really; we're not buffering */
}

static int zaucon_chars_in_buffer(struct tty_struct *tty)
{
	return 0;		/* we're not buffering */
}

static void zaucon_flush_chars(struct tty_struct *tty)
{
	;
}

static int zaucon_open(struct tty_struct *tty, struct file * filp)
{
	unsigned int	currcons;
	currcons = MINOR(tty->device) - tty->driver.minor_start;

//0214	printk ("console %d open\n", currcons);

	return 0;
}

static void zaucon_close(struct tty_struct *tty, struct file * filp)
{
	unsigned int	currcons;
	currcons = MINOR(tty->device) - tty->driver.minor_start;

//0214	printk ("console %d close\n", currcons);
}

struct tty_driver console_driver;
static int console_refcount;
static struct tty_struct* console_table[MAX_NR_CONSOLES];
static struct termios* zaucon_termios[MAX_NR_CONSOLES];
static struct termios* zaucon_termios_locked[MAX_NR_CONSOLES];

static void console_bh(void)
{
	run_task_queue(&con_task_queue);
}

void __init zaucon_init(void)
{
	unsigned int currcons = 0;

	memset(&console_driver, 0, sizeof(struct tty_driver));
	console_driver.magic = TTY_DRIVER_MAGIC;
	console_driver.name = "tty";
	console_driver.name_base = 1;
	console_driver.major = TTY_MAJOR;
	console_driver.minor_start = 1;
	console_driver.num = MAX_NR_CONSOLES;
	console_driver.type = TTY_DRIVER_TYPE_CONSOLE;
	console_driver.init_termios = tty_std_termios;
	console_driver.flags = TTY_DRIVER_REAL_RAW | TTY_DRIVER_RESET_TERMIOS;
	console_driver.refcount = &console_refcount;
	console_driver.table = console_table;
	console_driver.termios = zaucon_termios;
	console_driver.termios_locked = zaucon_termios_locked;

	console_driver.open = zaucon_open;
	console_driver.close = zaucon_close;
	console_driver.write = zaucon_write;
	console_driver.write_room = zaucon_write_room;
	console_driver.put_char = zaucon_put_char;
	console_driver.flush_chars = zaucon_flush_chars;
	console_driver.chars_in_buffer = zaucon_chars_in_buffer;
	console_driver.ioctl = NULL;
	console_driver.stop = NULL;
	console_driver.start = NULL;
	console_driver.throttle = NULL;
	console_driver.unthrottle = NULL;

	if (tty_register_driver(&console_driver))
		panic("Couldn't register zaurus console driver\n");

	init_bh(CONSOLE_BH, console_bh);
}
